function [excitPat, specLoudn] = CalcLoudnessFunctions(f, I, z, ...
    freeField, binaural)
% This software was written by Preeti Rao at the 
% Institute for Perception Research (IPO) in 1999, and 
% adapted by Dik Hermes, Human-Technology Interaction group,
% Eindhoven University of Technology in 2009.
%
% The function CalcLoudnessFunctions converts power spectral 
% lines to an E pattern "excitationPattern" and a 
% specific-loudness curve "specificLoudness".
% The input parameters are "f", the vector with the input 
% frequencies, "I", the spectral power vector specified as 
% input power in W/m^2.
% The vector "z" represents the ERB nrs for which the 
% E pattern and the specific loudness are 
% calculated. The boolean "freeField" will calculate the 
% far-field correction if true, while the boolean "binaural", 
% if true, assumes binaural listening.
% So, setting the listening conditions to "freeField = 1;" 
% and "binaural = 1;" gives the E pattern and 
% specific loudness for binaural listening in free 
% field conditions.  
% 
% The models incorporate the descriptions published in:
% 
% [1] Moore B.C.J. & Glasberg B.R. (1996)
% A revision of Zwicker's loudness Model.
% Acustica 82, 335-345.
% 
% [2] Moore B.C.J., Glasberg B.R. & Baer T. (1997), 
% A model for the prediction of thresholds, loudness, 
% and partial loudness. 
% J. Audio Eng. Soc. 45(4), 224-240.
% 
% The rounded exponential filter shapes are calculated 
% according to:
% [3] Glasberg B.R. & Moore B.C.J. (1990)
% Derivation of auditory filter shapes from notched-noise data.
% Hearing Research 47, 103-128.

% I_ref is the intensity in W/m2 corresponding to 0 db SIL
I_ref = 1.0e-12;    
nrOfPartials = length(f);
nZ = length(z);

% Set minimum input level for auditory filters
% This is only done to fix the shape of the auditory filter.
min_spl_dB = -500; 
min_spl = I_ref * 10^(min_spl_dB/10);

% Apply outer ear filtering in case of free field stimulation
if freeField
    I = OuterEarFilter(f, I);
end
% Apply middle ear filtering
I = MiddleEarFilter(f, I);

% Set minimum intensity of filters to min_spl dB SPL.
I = max(min_spl, I);
I_dB = 10*log10(I/I_ref);

% Calculate the frequencies of the z numbers in array z
% according to the inverse of Equation 2 in [2].
fErb = (10.^(z/21.4)-1)*1000/4.37;

% Calculate the contribution of each partial to 
% the excitation E at every location on the z array.
E = zeros(nZ, 1);
for k = 1:nrOfPartials
    w = RoexFiltering(fErb, f(k), I_dB(k)); 
    E = E + w' * I(k);
end
E = E';

% Now the excitation levels in E are converted from W/m^2 
% to dB SIL.
excitPat = 10*log10(E/I_ref);

% Calculate partial specific loudness from excitation.
specLoudn = ExcitationToSpecificLoudness(E, fErb);

% Add up the loudness from both ears for binaural stimulation.
if binaural
    specLoudn = 2*specLoudn;
end
