function [specificLoudness_n, specificLoudness_t, partialSpecificLoudness] = ...
    CalcPartialLoudnessFunctions(f, I_n, I_t, z, freeField, binaural)
% This software was written by Preeti Rao at the Institute for Perception
% Research (IPO) in 1999, and adapted by Dik Hermes, Human-Technology
% Interaction group, Eindhoven University of Technology in 2009.
%
% The function CalcSpecificLoudnessFunctions converts power spectral lines of
% the noise I_n and the signal plus noise I_t to excitation patterns and 
% based on these calculates the specific loudness of the noise, the specific
% loudness of the signal plus noise, and the partial specific loudness of the
% signal. The input parameters are f, the vector with the input frequencies,
% I_n and I_t, the spectral power vectors of the noise and the signal plus
% noise, resp., specified as input power in W/m^2. The vector z represents the
% ERB nrs for which the excitation patterns and the specific loudness are
% calculated. The boolean freeField will calculate the far-field correction if
% true, while the boolean binaural, if true, assumes binaural listening. The
% models incorporate the descriptions published in:
% 
% [1] Moore B.C.J. & Glasberg B.R. (1996) A revision of Zwicker's loudness
% Model. Acustica 82, 335-345.
% 
% [2] Moore B.C.J., Glasberg B.R. & Baer T. (1997) A model for the prediction
% of thresholds, loudness, and partial loudness. J. Audio Eng. Soc.45(4),
% 224-240.
% 
% The rounded exponential filter shapes are calculated according to: 
% [3] Glasberg B.R. & Moore B.C.J. (1990) Derivation of auditory filter shapes 
% from notched-noise data. Hearing Research 47, 103-128.

% I_ref is the intensity in W/m^2 corresponding to 0 db SIL.
I_ref = 1.0e-12; 
nrOfPartials = length(f);
nZ = length(z);

% Set minimum input level for auditory filters.
% This is only done to fix the shape of the auditory filter.
min_spl_dB = -500; 
min_spl = I_ref * 10^(min_spl_dB/10);

% Apply outer ear filtering in case of free field stimulation.
if freeField
    I_n = OuterEarFilter(f, I_n);
    I_t = OuterEarFilter(f, I_t);
end
% Apply middle ear filtering.
I_n = MiddleEarFilter(f, I_n);
I_t = MiddleEarFilter(f, I_t);

% Set minimum excitation of filters to min_spl dB SPL.
I_t = max(min_spl, I_t);
I_t_dB = 10*log10(I_t/I_ref);

% Calculatate the frequencies of the z numbers in array z according to the
% inverse of Equation 2 in [2].
fErb = (10.^(z/21.4)-1)*1000/4.37;

% Calculate the contribution of each partial to the excitation at every location
% on the z array.
E_n = zeros(nZ, 1);
E_t = zeros(nZ, 1);
for k=1:nrOfPartials
    w = RoexFiltering(fErb, f(k), I_t_dB(k));
    E_n = E_n + w' * I_n(k);
    E_t = E_t + w' * I_t(k);
end
E_n = E_n';
E_t = E_t';

% Define the signal and noise excitations.
E_s = max(abs(E_t - E_n), 10^(min_spl_dB));  
E_n = min(E_n, E_t);

% Calculate partial specific loudness from excitation.
partialSpecificLoudness = ExcitationToPartialSpecificLoudness(E_s, E_n, fErb);
specificLoudness_n = ExcitationToSpecificLoudness(E_n, fErb);
specificLoudness_t = ExcitationToSpecificLoudness(E_t, fErb);

% Add up the loudness from both ears for binaural stimulation.
if binaural
    partialSpecificLoudness = 2*partialSpecificLoudness; 
    specificLoudness_n = 2*specificLoudness_n; 
    specificLoudness_t = 2*specificLoudness_t; 
end 
