function roughness = CalcRoughness(segment, sampleFrequency)

% This function calculates the roughness of a 200 ms sound segment
% sampled with sample frequency 'sampleFrequency'.
% It is based on the algorithm presented by:
% Daniel, P.; Weber, R. (1997)
% Psychoacoustical roughness: Implementation of an optimized model.
% Acta Acustica united with Acustica 83.1: 113-123.
% It is a revision and correction of the software written by Jasper 
% Schrader:
% Schrader, Jasper (2002)
% A MATLAB implementation of a model of auditory roughness.
% Eindhoven: Eindhoven University of Techonology, 2002, MSc thesis.
% An essential correction is made in line 50 in which the slope of the
% excitations are calculated. In the original version by Jasper Schrader
% line 50 run: 
% slopeHigh(k) = min(-24-(230./freqs(k+4))+(0.2*dBSpc(iComp(k))), 0);
% and the script was used with another table to calculate the g factors 
% (see CalcGFactors).

% Calculate Blackman analysis window
% Calibrate for wav-level and intensity-level
n = length(segment);
window = blackman(n)';
window = window/sum(window);
% Apply analysis window to segment
segment = window.*segment;

% Calculate transfer characteristic of the outer and inner ear
% Calculate function variables
segmentDur = n/sampleFrequency;
nMax = round(sampleFrequency/2*segmentDur);
nZ = 1:nMax;
freqs = nZ/segmentDur;
barks(nZ) = Hz2Bark(freqs);
transferOuterInnerEar = zeros(1,n);
transferOuterInnerEar(nZ) = CalcTransferOuterInnerEar(barks(nZ));

% Make a list of minimum excitation
% and find the spectral components higher than this
minExcitDB(nZ) = CalcMinExcitation(barks(nZ));
complexSpc = transferOuterInnerEar.*fft(segment);
amplSpc = abs(complexSpc(nZ));
dBSpc = mag2db(amplSpc);
iComp = find(dBSpc>minExcitDB);
nComp = length(iComp);

% Calculate the slopes of the excitations by the significant components
% Terhardt (1979), p. 158, Eqs 2 and 3;
% Daniel & Weber (1997), p. 117, Eqs 4 and 5.
slopeLow = -27;
slopeHigh = zeros(1, nComp);
k = 1:nComp;
slopeHigh(k) = min(-24-(230./freqs(iComp(k)))+(0.2*dBSpc(iComp(k))), 0);

% Calculate excitation pattern for 47 auditory filters separated half a 
% Bark on a Bark scale 
nCh = 47;
z = (1:nCh)/2;
zb = Bark2Hz(z)*segmentDur;
minExcitDB = interp1(nZ, minExcitDB, zb);
chLow = floor(2*barks(iComp));
chHgh = ceil(2*barks(iComp));
slopes = zeros(nComp, nCh);
for k = 1:nComp
    levDB = dBSpc(iComp(k));
    b = barks(iComp(k));
    for j = 1:chLow(k)
        sl = (slopeLow*(b-(j*0.5)))+levDB;
        if sl>minExcitDB(j)
            slopes(k, j) = db2mag(sl);
        end
    end
    for j = chHgh(k):nCh
        sl = (slopeHigh(k)*((j*0.5)-b))+levDB;
        if sl>minExcitDB(j)
            slopes(k, j) = db2mag(sl);
        end
    end
end

% Calculate weighting functions h (Daniel & Weber (1997), p.117) 
hWeight = Hweights(n, sampleFrequency);
% Calculate modulation depth of the temporal envelopes of 
% the auditory-filter outputs
temporalEnvelope = zeros(nCh, n);
modDepth = zeros(1, nCh);
for k = 1:nCh
    exc = zeros(1, n);
    for j = 1:nComp
        i = iComp(j);
        if chLow(j) == k
            ampl = 1;
        elseif chHgh(j) == k
            ampl = 1;
        elseif chHgh(j)>k
            ampl = slopes(j, k+1)/amplSpc(i);
        else
            ampl = slopes(j, k-1)/amplSpc(i);
        end
        exc(i) = ampl*complexSpc(i);
    end
    excitation = abs(n*real(ifft(exc)));
    h0 = mean(excitation);
    spcEnvelope = fft(excitation-h0);
    temporalEnvelope(k,:) = 2*real(ifft(spcEnvelope.*hWeight(k,:)));
    rmsTempEnv = rms(temporalEnvelope(k,:));
    if h0>0
        modDepth(k) = rmsTempEnv/h0;
        if modDepth(k)>1
            modDepth(k) = 1;
        end
    else
        modDepth(k) = 0;
    end
end

% Calculate correlation coefficients 
% between temporal envelopes of      
% auditory filters 1 Bark apart      
cCoefs = zeros(1, nCh-2);
for k = 1:nCh-2
    cc = corrcoef(temporalEnvelope(k,:), temporalEnvelope(k+2,:));
    if ~isnan(cc)
        cCoefs(k) = cc(2,1);
    end
end

% Calculate specific roughness 
gFactors = CalcGFactors(z);
specRoughness = zeros(1, nCh);
specRoughness(1) = gFactors(1)*(modDepth(1)*cCoefs(1))^2;
specRoughness(2) = gFactors(2)*(modDepth(2)*cCoefs(2))^2;
k = 3:nCh-2;
specRoughness(k) = ...
    gFactors(k).*(modDepth(k).*cCoefs(k-2).*cCoefs(k)).^2;
specRoughness(nCh-1) = ...
    gFactors(nCh-1)*(modDepth(nCh-1)*cCoefs(nCh-3))^2;
specRoughness(nCh) = gFactors(nCh)*(modDepth(nCh)*cCoefs(nCh-2))^2;

% Calculate total roughness with a calibration factor 'cal' to produce 
% a roughness of 1 asper for a 1-kHz, 60-dB tone with carrier frequency 
% of 70 Hz and a modulation depth of 1 (Daniel & Weber (1997), p. 118).
cal = 0.25; 
roughness = cal*sum(specRoughness);

