
function [b, a] = CalculateGammatoneFilterBank(sampleFrequency, ...
    centerFrequencies)

% CalculateGammatoneFilterBank computes the filter coefficients for a bank 
% of Gammatone filters. These filters were proposed by Patterson et al. 
% (1976) for simulating the auditory filters. The results are returned 
% as forward and backward arrays of filter coefficients. The array 
% "centerFrequencies" must contain the center frequencies of the auditory 
% filters.
%

% Calculate in radians the auditory filter bandwidths using Equation 3 of 
% Glasberg and Moore (1990) "Derivation of auditory filter shapes 
% from notched-noise data", Hearing Research 47: 103-138.
bandWidths = ...
    2*pi*24.7*(0.00437*centerFrequencies + 1); % radians
dt = 1/sampleFrequency;

% The following is adapted from the gammaToneToolKit, downloaded 
% from http://auditoryneuroscience.com/travellingWave, which is based on 
% Slaney (1993) "An efficient implementation of the Patterson-Holdsworth 
% cochlear filter bank", Apple Computer Technical Report #35.
p = 2*pi*centerFrequencies*dt;
q = 1i*p;
exp2ip = exp(2*q);
r = exp(bandWidths*dt);
v = -2*exp2ip*dt;
w = 2*exp(-(bandWidths*dt)+q)*dt;
cos1p = cos(p);
cos2p = cos(2*p);
cos3p = cos(3*p);
cos4p = cos(4*p);
sin1p = sin(p);
dt4 = dt^4;
gain = ...
    abs((v + w .* (cos1p-sqrt(3-2^(3/2))*sin1p)) .* ...
        (v + w .* (cos1p+sqrt(3-2^(3/2))*sin1p)) .* ...
        (v + w .* (cos1p-sqrt(3+2^(3/2))*sin1p)) .* ...
        (v + w .* (cos1p+sqrt(3+2^(3/2))*sin1p)) ./ ...
        (-2 ./ (r.^2) - 2*exp2ip + 2*(1 + exp2ip) ./ r).^4);
b = zeros(length(centerFrequencies),5);
a = zeros(length(centerFrequencies),9);
b(:,1) =  dt4 ./ gain;
b(:,2) = -4*dt4*cos1p ./ (r.^1) ./ gain;
b(:,3) =  6*dt4*cos2p ./ (r.^2) ./ gain;
b(:,4) = -4*dt4*cos3p ./ (r.^3) ./ gain;
b(:,5) =    dt4*cos4p ./ (r.^4) ./ gain;
a(:,1) = ones(length(centerFrequencies),1);
a(:,2) = -8*cos1p ./ r;
a(:,3) =  4*(4 + 3*cos2p) ./ (r.^2);
a(:,4) = -8*(6*cos1p + cos3p) ./ (r.^3);
a(:,5) =  2*(18 + 16*cos2p + cos4p) ./ (r.^4);
a(:,6) = -8*(6*cos1p + cos3p) ./ (r.^5);
a(:,7) =  4*(4 + 3*cos2p) ./ (r.^6);
a(:,8) = -8*cos1p./(r.^7);
a(:,9) = (r.^(-8));

