function partialSpecificLoudness = ...
    ExcitationToPartialSpecificLoudness(E_s, E_n, f)
% This software was written by Preeti Rao at the Institute for Perception
% Research (IPO) in 1999, and adapted by Dik Hermes, Human-Technology
% Interaction group, Eindhoven University of Technology in 2009.
%
% The function ExcitationToSpecificLoudness calculates the partial specific
% loudness partialSpecificLoudness from the excitation patterns E_s of the
% signal and that of the noise E_n, for the frequencies in f. It implements Eqs
% 17, 18, 19 and 20 of: Moore B.C.J., Glasberg B.R. && Baer T. (1997), A model
% for the prediction of thresholds, loudness, and partial loudness. J. Audio
% Eng. Soc.45(4), 224-240.
% 
% I_ref is the intensity in W/m^2 corresponding to 0 db SPL E_thr_ref_dB is the
% internal noise excitation in dB above 0.5 kHz. C is the constant C in the Eqs
% for N' in Moore, Glasberg, and Baer (1997), where it is 0.047.  It is adjusted
% in such a way that a binaural, free-field pure tone of 1000 Hz and 40 dB is
% exactly 1 sone.  

I_ref = 1.0e-12;
E_thr_ref_dB = 3.6;
C = 0.0467338323242624;
C_2 = C/((1.04*10^6)^0.5);

% Calculate E_thr_Q and G for f.
E_thr_Q_dB = InternalExcitation(f);
E_thr_Q = 10.^(E_thr_Q_dB/10);
G_dB = -(E_thr_Q_dB-E_thr_ref_dB);
G = 10.^(G_dB/10);
% Calculate alpha and A for f from G_dB.
[alpha, A] = AlphaAFromG_dB(G_dB);

% Calculate K parameters.
K_dB = CalcK_Parameter(f);

% Apply equation in text of [1] just below Equation 15.
E_thr_n_dB = K_dB+10*log10(E_n/I_ref);
E_thr_n = 10.^(E_thr_n_dB/10);
E_thr_n = E_thr_n + E_thr_Q;

% Normalize E_s and E_n.
E_s = E_s/I_ref;
E_n = E_n/I_ref;

partialSpecificLoudness = zeros(size(f));
for k = 1:length(f)
    % Equation 17
    if((E_s(k) >= E_thr_n(k)) && (E_s(k)+E_n(k) <= 10^10))
        partialSpecificLoudness(k) = ...
            C*((G(k)*(E_s(k)+E_n(k)) + A(k))^alpha(k) - A(k)^alpha(k));
        partialSpecificLoudness(k) = partialSpecificLoudness(k) - ...
            C*(E_thr_n(k)/E_s(k))^0.3 * ...
            (G(k)*(E_n(k)+E_thr_n(k))+ A(k))^alpha(k);
        partialSpecificLoudness(k) = partialSpecificLoudness(k) + ...
            C*(E_thr_n(k)/E_s(k))^0.3 * ...
            (G(k)*E_thr_Q(k)+A(k))^alpha(k);
    end

    % Equation 18
    if ((E_s(k) < E_thr_n(k)) && (E_s(k)+E_n(k) <= 10^10))
        partialSpecificLoudness(k) = ...
            ((G(k)*E_thr_Q(k)+A(k))^alpha(k)-A(k)^alpha(k))/ ...
            ((G(k)*(E_n(k)+E_thr_n(k)) + ...
            A(k))^alpha(k)-(G(k)*E_n(k)+A(k))^alpha(k));
        partialSpecificLoudness(k) = partialSpecificLoudness(k) * ...
            C*(2*E_s(k)/(E_s(k)+E_thr_n(k)))^1.5;
        partialSpecificLoudness(k) = partialSpecificLoudness(k) * ...
            ((G(k)*(E_s(k)+E_n(k))+A(k))^alpha(k) - ...
            (G(k)*E_n(k)+A(k))^alpha(k));
    end

    % Equation 19
    if ((E_s(k) >= E_thr_n(k)) && ((E_s(k)+E_n(k)) > 10^10))
        partialSpecificLoudness(k) = ...
            -C_2 * ((E_n(k)+E_thr_n(k))^0.5 - ...
            (G(k)*E_thr_Q(k)+A(k))^alpha(k) + A(k)^alpha(k));
        partialSpecificLoudness(k) = partialSpecificLoudness(k) * ...
            (E_thr_n(k)/E_s(k))^0.3;
        partialSpecificLoudness(k) = partialSpecificLoudness(k) + ...
            C_2*(E_s(k)+E_n(k))^0.5;
    end

    % Equation 20
    if ((E_s(k) < E_thr_n(k)) &&  ((E_s(k)+E_n(k)) > 10^10))
        partialSpecificLoudness(k) = ((G(k)*E_thr_Q(k)+A(k))^alpha(k) - ...
            A(k)^alpha(k))/((E_n(k)+E_thr_n(k))^0.5 - E_n(k)^0.5);
        partialSpecificLoudness(k) = ...
            C*(2*E_s(k)/(E_s(k)+E_n(k)))^1.5*partialSpecificLoudness(k);
        partialSpecificLoudness(k) = ...
            ((E_s(k)+E_n(k))^0.5 - E_n(k)^0.5)*partialSpecificLoudness(k);
    end
end
   
