function specificLoudness = ExcitationToSpecificLoudness(E, f)
% This software was written by Preeti Rao at the Institute for Perception
% Research (IPO) in 1999, and adapted by Dik Hermes, Human-Technology
% Interaction group, Eindhoven University of Technology in 2009.
%
% The function ExcitationToSpecificLoudness calculates the specific loudness
% specificLoudness from the excitation pattern E for the frequencies in f. It
% implements Eqs 6, 8, and 9 of: 
% Moore B.C.J., Glasberg B.R. & Baer T. (1997) A model for the prediction of
% thresholds, loudness, and partial loudness. J. Audio Eng. Soc.45(4), 224-240.
% 
% I_ref is the intensity in W/m^2 corresponding to 0 db SPL. E_thr_ref_dB is the
% internal noise E in dB above 0.5 kHz. C is the constant C in the Eqs for N' in
% Moore, Glasberg, and Baer (1997), where it is 0.047.  It is adjusted in such a
% way that a binaural, freefield pure tone of 1000 Hz and 40 dB is exactly 1
% sone.  
I_ref = 1.0e-12;
E_thr_ref_dB = 3.6;
C = 0.0467338323242624;

% Calculate E_thr_Q and G for f.
E_thr_Q_dB = InternalExcitation(f);
G_dB = -(E_thr_Q_dB-E_thr_ref_dB);
G = 10.^(G_dB/10);
% Calculate alpha and A for f from G_dB.
[alpha, A] = AlphaAFromG_dB(G_dB);

% Equation 6
E_s = E/I_ref;
specificLoudness = C*((G.*E_s+A).^alpha - A.^alpha);

E_thr_Q = zeros(size(E_thr_Q_dB));
for k = 1:length(f)
    % Equation 8
    % Adaptation for low excitation values
    E_thr_Q(k) = 10^(E_thr_Q_dB(k)/10);
    if E_s(k) < E_thr_Q(k)
        specificLoudness(k) = ...
            specificLoudness(k) * ...
            (2*E_s(k)/(E_s(k)+E_thr_Q(k)))^1.5;
    end

    % Equation 9
    % Adaptation for high excitation values
    if E_s(k) > 10^10
        specificLoudness(k) = C*(E_s(k)/1.04e+6)^0.5;
    end
end
