function excitation = RoexFiltering(fRange, fCenter, fLevel)
% This software was written by Preeti Rao at the 
% Institute for Perception Research (IPO) in 1999, and 
% adapted by Dik Hermes, Human-Technology Interaction group,
% Eindhoven University of Technology in 2009 and 2016.
% 
% The function RoexFiltering computes the excitation pattern
% of a sound component with frequency fCenter and level fLevel 
% for the range of frequencies speified by fRange. It is assumed 
% that the auditory filter can be modelled as a so-called 
% rounded exponential (roex) filter the frequency response of 
% which can be represented as (1+p*g)*exp(-p*g), in which 
% g is normalized frequency and p represents the slope of the 
% filter. The minumum input of each filter is set at 
% minLevel, 20 dB.
% The rounded exponential filter shapes are calculated
% according to:
%
% [1] Moore B.C.J. & Glasberg B.R. (1987) Formulae describing
% frequency selectivity as a function of frequency and
% level, and their use in calculating excitation patterns.
% Hearing Research 28, 209-225.
%
% [2] Glasberg B.R. & Moore B.C.J. (1990)
% Derivation of auditory filter shapes from notched-noise data.
% Hearing Research 47, 103-128.
% 
% [3] Moore B.C.J., Glasberg B.R. & Baer T. (1997)
% A model for the prediction of thresholds, loudness, 
% and partial loudness. 
% Journal of the Audio Engineering Society 45.4: 224-240.

minLevel = 20; % dB
fRangekHz = fRange/1000;
fLevel = max(minLevel, fLevel);

% Calculate the ERB at frequencies fRangekHz according
% to Eq. 3 in [2]
fRangeErb = 24.7*(4.37*fRangekHz+1);

% Calculation of the upper and lower p-values, pHigh and pLow
f1kHz = 1;
erb1kHz = 24.7*(4.37*f1kHz+1); % ERB for 1 kHz
p1kHz51 = (4*1000)/erb1kHz; % p at 1 kHz for 51 dB, where pu = pl.
p51 = 4*fRange./fRangeErb;
% Next Eq. 8 in [1] or Eq. 5 in [2] are applied with 0.38 
% replaced by 0.35 following [3].  Moreover, [3] supposes 
% pHigh to be constant with level. 
pLow = p51-0.35*(fLevel-51)*(p51/p1kHz51);
pHigh = p51;

% pl and pu cannot be negative
pLow = max(0, pLow);
pHigh = max(0, pHigh);

% Calculate the normalized frequencies
g = (fCenter-fRange)./fRange;
ineg = find(g<0);
ipos = find(g>=0);

% Calcultate excitation
excitation(ineg) = ...
    (1-pLow(ineg).*g(ineg)).*exp(+pLow(ineg).*g(ineg));
excitation(ipos) = ...
    (1+pHigh(ipos).*g(ipos)).*exp(-pHigh(ipos).*g(ipos));
