function w = RoundedExponential(fRange, fCenter, fLevel)
% This software was written by Preeti Rao at the 
% Institute for Perception Research (IPO) in 1999, and 
% adapted by Dik Hermes, Human-Technology Interaction group,
% Eindhoven University of Technology in 2009 and 2016.
% 
% The function RoundedExponential computes the intensity transfer
% function of a "roex" filter. The scalar fCenter represents 
% the filter center frequency and fLevel its intensity 
% level.  The vector fRange contains the frequencies for 
% which the output of the roex-filter is calculated.
% A roex filter can be represented as (1+p*g)*exp(-p*g), in which g 
% is normalized frequency and p represents the slope of the 
% filter. The minumum input of each filter is set at 
% minLevel, 20 dB
% The rounded exponential filter shapes are calculated
% according to:
%
% [1] Moore B.C.J. & Glasberg B.R. (1987) Formulae describing
% frequency selectivity as a function of frequency and
% level, and their use in calculating excitation patterns.
% Hearing Research 28, 209-225.
%
% [2] Glasberg B.R. & Moore B.C.J. (1990)
% Derivation of auditory filter shapes from notched-noise data.
% Hearing Research 47, 103-128.
% 
% [3] Moore B.C.J., Glasberg B.R. & Baer T. (1997)
% A model for the prediction of thresholds, loudness, 
% and partial loudness. 
% J. Audio Eng. Soc. 45(4), 224-240.

minLevel = 20;
fCenterkHz = fCenter/1000;
fLevel = max(minLevel, fLevel);

% Calculate the ERB at frequency "fk" according
% to Eq. 3 in [2]
fCenterErb = 24.7*(4.37*fCenterkHz+1);

% Calculation of the upper and lower p-values, pu and pl
f1kHz = 1;
erb1kHz = 24.7*(4.37*f1kHz+1); % ERB for 1 kHz
p1kHz51 = (4*1000)/erb1kHz; % p at 1 kHz for 51 dB, where pu = pl.
p51 = 4*fCenter/fCenterErb;
% Next Eq. 8 in [1] or Eq. 5 in [2] are applied with 0.38 replaced 
% by 0.35 following [3].  Moreover, [3] supposes pu to be constant
% with level. 
pLow = p51-0.35*(fLevel-51)*(p51/p1kHz51);
pHigh = p51;

% pl and pu cannot be negative
pLow = max(0, pLow);
pHigh = max(0, pHigh);

% Calculate the normalized frequencies
g = (fRange-fCenter)/fCenter;
ineg = find(g<0);
ipos = find(g>=0);

% Calcultate transfer
w = zeros(size(g));
w(ineg) = (1-pLow*g(ineg)).*exp(+pLow*g(ineg));
w(ipos) = (1+pHigh*g(ipos)).*exp(-pHigh*g(ipos));

