clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Read the speech signal
[speech, sampleFrequency] = audioread('allLines.wav');
speech = 0.3*speech';
% Specify the noise bursts
cutOffMin = 100;
cutOffMax = 4900;
durationInterval = 0.2; % sec
durationBurst = 0.1; % sec
fade = 0.01; % sec

% Calculate the stimulus
dt = 1/sampleFrequency;
durationStimulus = length(speech)*dt; % sec
t = dt:dt:durationStimulus;
tInterval  = dt:dt:durationInterval;
interval = zeros(size(tInterval));
tNoise = dt:dt:durationBurst;
% Generate the envelope
env = FadeInOut(ones(size(tNoise)), sampleFrequency, fade, fade);
nrOfRepetitions = ceil(durationStimulus/(durationInterval+durationBurst));
envelope = repmat([interval env], 1, nrOfRepetitions);
% Generate the noise
w = [2*cutOffMin/sampleFrequency 2*cutOffMax/sampleFrequency];
[b, a] = butter(4, w);
noise = 0.2*filtfilt(b, a, randn(size(speech)));
% Generate the interrupted speech, the noise bursts and the combination
noiseBursts = noise.*envelope(1:length(speech));
interruptedSpeech = speech.*(1-envelope(1:length(speech)));
stimContIll = interruptedSpeech + noiseBursts;
stimulus = [interruptedSpeech interval noiseBursts interval stimContIll];

% Save and play the stimulus
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [90 150 150 150]; % pixels
xLabelHeight = 70; % pixels
yLabelWidth = 70; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the speech signal
panel = 1;
axes('Position', pos(panel,:))
tAxis = [t(1) t(end)];
maxYAxis = sampleFrequency/2; % Hz
fAxis = [0 maxYAxis];
dXTick = 0.5; % sec
dYTick = 1; % kHz
PlotSignal(stimContIll, sampleFrequency, ...
    tAxis, dXTick, blueGreen, 'time (s)')
set(gca, 'XTick', 0:dXTick:durationStimulus, ...
    'XTickLabel', 0:dXTick:durationStimulus, ...
    'Color', lightYellowGreen)
syllables = ...
    ['All  '; 'lines'; 'from '; 'Lon -'; 'don  '; 'are  '; ...
    'en  -'; 'gaged'];
onsets = [0.13 0.34 0.70 0.90 1.10 1.25 1.40 1.60];
for syll = 1:length(syllables)
    text(onsets(syll), -0.8, syllables(syll,:), ...
        'FontAngle', 'italic', ...
        'FontSize', 10, 'FontWeight', 'bold', 'Color', blueGreen)
end

% Plot the spectrogram of the sum of the bursts and the interrupted speech
panel = panel+1;
axes('Position', pos(panel,:))
nFft = 256; % Number of samples in fft window
nWindow = 64;
nOverlap = nWindow-32;
[~, f, ts, spgr] = spectrogram(stimContIll, ...
    nWindow, nOverlap, nFft, sampleFrequency);
spgr = 10*log10(abs(spgr));
PlotSpectrogram(speech, sampleFrequency, ...
    f, ts, spgr, tAxis, dXTick, fAxis, dYTick, 40, ...
    frameColor)
set(gca, 'Color', lightYellowGreen)
ylabel('freq. (kHz)')

% Plot the spectrogram of the bursts
panel = panel+1;
axes('Position', pos(panel,:))
[~, f, ts, spgr] = spectrogram(noiseBursts, ...
    nWindow, nOverlap, nFft, sampleFrequency);
spgr = 10*log10(abs(spgr));
PlotSpectrogram(speech, sampleFrequency, ...
    f, ts, spgr, tAxis, dXTick, fAxis, dYTick, 40, ...
    frameColor)
set(gca, 'Color', lightYellowGreen)
ylabel('freq. (kHz)')

% Plot the spectrogram of the interrupted speech
panel = panel+1;
axes('Position', pos(panel,:))
[~, f, ts, spgr] = spectrogram(interruptedSpeech, ...
    nWindow, nOverlap, nFft, sampleFrequency);
spgr = 10*log10(abs(spgr));
PlotSpectrogram(speech, sampleFrequency, ...
    f, ts, spgr, tAxis, dXTick, fAxis, dYTick, 40, ...
    frameColor)
set(gca, 'Color', lightYellowGreen)
ylabel('freq. (kHz)')

