clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Define the stimulus
sampleFrequency = 10000; % Hz
dt = 1/sampleFrequency;
durationVowel = 5; % sec
tVowel = dt:dt:durationVowel;
silenceDuration = 0.2;
silence = zeros(size(dt:dt:silenceDuration));
ampl = 0.9;
f0 = 150; % Hz
% Formant frequencies and bandwidth of /a/
formantFrequencies = [890 1360 2500 3500 4500];
formantBandwidths = [40 60 80 120 180];

% Synthesize pulses with micro-modulation
noise = randn(size(tVowel));
cutoffLow = 3; % Hertz
w = 2*cutoffLow/sampleFrequency;
[b, a] = butter(4, w, 'low');
lpNoise = filter(b, a, noise);
lpNoise = (lpNoise-mean(lpNoise))/max(abs(lpNoise));
modInd = 3;
modFreq = 2;
instFreq = f0+modInd*modFreq*lpNoise;
pulsesNoMod = zeros(size(tVowel));
pulsesNoMod(1:round(sampleFrequency/f0):length(tVowel)) = 1;
phi = 2*pi*cumsum(instFreq)*dt;
pulsesMod = [0 diff(square(phi))>0];
pulsesMod = FadeInOut(pulsesMod, sampleFrequency, 0.02, 0.02);
pulsesNoMod = FadeInOut(pulsesNoMod, sampleFrequency, 0.02, 0.02);

% Software for vowel synthesis adapted from: https://
% nl.mathworks.com/matlabcentral/fileexchange/45449-vowel-synthesis

% Synthesize Rosenberg pulse
samplesInPeriod  =  round(sampleFrequency/f0);
p1 = 0.25;
p2 = 0.10;
n1 = round(samplesInPeriod*p1);
n2 = round(samplesInPeriod*p2);
i1 = 0:n1;
glottalPulse = 0.5*(1-cos(pi*i1/n1));
i2 = 1:n2;
glottalPulse = [glottalPulse cos(pi*i2/(2*n2))];
glottalPulse = ...
    [glottalPulse zeros(1, samplesInPeriod-length(glottalPulse))];

% Convolve glottal pulse with pulse series
sourceMod = conv(glottalPulse, pulsesMod);
sourceNoMod = conv(glottalPulse, pulsesNoMod);

% Calculate formant filter and filter source
dw = 2*pi*dt;
a = 1;
b = 1;
for k = 1:length(formantFrequencies)
    p = exp(-formantBandwidths(k)*dw);
    q = cos(formantFrequencies(k)*dw);
    a = conv(a, [1 -2*p*q p*p]);
    b = conv(b, 1-2*p*q+p*p);
end
sourceMod = filter(b, a, sourceMod);
sourceNoMod = filter(b, a, sourceNoMod);
vowelMod = sourceMod;
vowelNoMod = sourceNoMod;
vowelMod = ampl*vowelMod/max(abs(vowelMod));
vowelNoMod = ampl*vowelNoMod/max(abs(vowelNoMod));
stimulus = [silence vowelNoMod silence vowelMod silence];
ins = zeros(1, length(glottalPulse)-1); % correction for convolution
instFreq = ...
    [f0*ones(size(instFreq)) ins silence instFreq ins silence];
t = dt:dt:length(instFreq)*dt;
    
% Save and play the sound
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Specify the figure parameters
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = 180; % pixels
xLabelHeight = 70; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
% Calculate the position of the figure on the screen
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Calculate the plot parameters
dXTick = 1; % sec
dYTick = 100; % Hz
tAxis = [-silenceDuration length(instFreq)*dt];
fAxis = [0 250];

% Plot the instantaneous fundamental frequency
panel = 1;
axes('Position', pos(panel,:))
set(gca, 'LineWidth', 2, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:length(instFreq)*dt, ...
    'YTick', 0:dYTick:sampleFrequency/2)
i = find(instFreq>0);
i1 = i(1);
i2 = i(1)+length(tVowel)-1;
plot(t(i1:i2), instFreq(i1:i2), 'LineWidth', 1.5, 'Color', blueGreen)
i1 = i(i2+1);
i2 = i(end);
plot(t(i1:i2), instFreq(i1:i2), 'LineWidth', 1.5, 'Color', blueGreen)
xlabel('time (s)', 'FontWeight', 'bold')
ylabel('{\it{F}}_{0} (Hz)', 'FontWeight', 'bold')
axis([tAxis fAxis])

    sprintf('mImages/%s.jpg', mfilename);
