clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Define the stimulus
sampleFrequency = 20000; % Hz
dt = 1/sampleFrequency;
durationVowel = 2; % sec
t = dt:dt:durationVowel;
ampl = 0.5;
f0 = 110; % Hz
% Formant frequencies and bandwidth of /a/
formantFrequencies = [890 1360 2500 3500 4500];
formantBandwidths = [40 60 80 120 180];
% Define the melody
freq = logspace(log10(f0), log10(2*f0), 13);
scale = [1 3 5 6 8 10 12 13];
samplesInTone = round(length(t)/length(scale));
for i = 1:length(scale)
    instFreq((i-1)*samplesInTone+1:i*samplesInTone) = freq(scale(i));
end
smoothWindow = 0.05;
h = hanning(round(smoothWindow*sampleFrequency));
instFreq = filtfilt(h/sum(h), 1, instFreq);

% Synthesize pulses with micro-modulation
phi = 2*pi*cumsum(instFreq)*dt;
pulses = [0 diff(square(phi))>0];
pulses = FadeInOut(pulses, sampleFrequency, 0.02, 0.02);

% Software for vowel synthesis adapted from: https://
% nl.mathworks.com/matlabcentral/fileexchange/45449-vowel-synthesis

% Synthesize Rosenberg pulse
samplesInPeriod = round(sampleFrequency/f0);
p1 = 0.25;
p2 = 0.10;
n1 = round(samplesInPeriod*p1);
n2 = round(samplesInPeriod*p2);
i1 = 0:n1;
glottalPulse = 0.5*(1-cos(pi*i1/n1));
i2 = 1:n2;
glottalPulse = [glottalPulse cos(pi*i2/(2*n2))];
glottalPulse = ...
    [glottalPulse zeros(1, samplesInPeriod-length(glottalPulse))];

% Convolve glottal pulse with pulse series
sourceMod = conv(glottalPulse, pulses);

% Calculate formant filter and filter source
dw = 2*pi*dt;
a = 1;
b = 1;
for k = 1:length(formantFrequencies)
    p = exp(-formantBandwidths(k)*dw);
    q = cos(formantFrequencies(k)*dw);
    a = conv(a, [1 -2*p*q p*p]);
    b = conv(b, 1-2*p*q+p*p);
end
sourceMod = filter(b, a, sourceMod);
vowelMod = sourceMod;
vowelMod = ampl*vowelMod/max(abs(vowelMod));
stimulus = vowelMod;
    
% Save and play the sound
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Specify the figure parameters
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = 180; % pixels
xLabelHeight = 70; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels

% Calculate the position of the figure on the screen
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the instantaneous fundamental frequency
axes('Position', pos)
% Calculate the plot parameters
dXTick = 0.5; % sec
YTick = [100 150 200]; % Hz
tAxis = [-0.1 t(end)+0.1];
fAxis = [90 260];
set(gca, 'LineWidth', 2, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:length(instFreq)*dt, ...
    'YTick', YTick, 'YScale', 'log')
plot(t, instFreq, 'LineWidth', 1.5, 'Color', blueGreen)
xlabel('time (s)', 'FontWeight', 'bold')
ylabel('{\it{F}}_{0} (Hz)', 'FontWeight', 'bold')
axis([tAxis fAxis])

    sprintf('mImages/%s.jpg', mfilename);
