clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Define the stimuli
sampleFrequency = 44100; % Hz
dt = 1/sampleFrequency;
durationTone = 0.5; % sec
durationInterval = 0.25; % sec
amp = 0.98;
fade = 0.02; % sec
deviation = 0.07; % relative deviation from harmonicity
modFreq = [200 240 300 400 600].*(1+deviation*randn(1, 5)); % Hz
n = length(modFreq);
fc = 1200; % Hz
modDepth = 600; % Hz

% Print the modulation frequencies
period = 1./modFreq;
fprintf('Modulation frequencies: \n')
fprintf('%8.2f', fliplr(modFreq))
fprintf('%s\n', '  Hz')

% Calculate the stimuli
% shift signal to get signal for t<0
t = -0.1*durationTone :dt:0.9*durationTone;
s = zeros(n, length(t));
instFreq = zeros(n, length(t));
interval = zeros(size(dt:dt:durationInterval));
stimulus = [];
mi = zeros(1,n);
for i = n:-1:1 % Start stimulus with highest modulation frequency
    mi(i) = modDepth/modFreq(i);
    instFreq(i,:) = fc+mi(i)*modFreq(i)*cos(2*pi*modFreq(i)*t);
    s(i,:) = amp*cos(2*pi*cumsum(instFreq(i,:))*dt);
    s(i,:) = FadeInOut(s(i,:), sampleFrequency, fade, fade);
    stimulus = [stimulus s(i,:) interval]; %#ok<AGROW>
end
stimulus = [stimulus interval interval];
for i = n:-1:1 % Start stimulus with highest modulation frequency
    tone = amp*sin(2*pi*modFreq(i)*t);
    tone = FadeInOut(tone, sampleFrequency, fade, fade);
    stimulus = [stimulus tone interval]; %#ok<AGROW>
end

% Calculate the spectra
nrOfSidebands = 25;
spc = zeros(size(modFreq, 1), 2*nrOfSidebands+1);
for i = 1:n
    for j = -nrOfSidebands:nrOfSidebands
        spc(i, nrOfSidebands+j+1) = amp*abs(besselj(j, mi(i)));
    end
end

% Save and play the stimulus
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, 0.5*stimulus, sampleFrequency)
sound(0.5*stimulus, sampleFrequency)

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = [480 225]; % pixels
panelHeights = 120*ones(1, n); % pixels
xLabelHeight = 70; % pixels
yLabelWidth = [70 70]; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the signals
for i = 1:n
    axes('Position', pos(i,:))
    tAxis = [-0.125*period(1) 2.5*period(1)];
    dXTick = 0.0025; % sec
    xTick = -dXTick:dXTick:durationTone;
    xTickLabel = xTick;
    yAxis = [-1.5 1.5];
    yTick = -2:2;
    set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
        'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
        'FontWeight', 'bold', 'TickDir', 'out', 'YTick', yTick)
    if i == 1
        set(gca, 'XTick', xTick, 'XTickLabel', 1000*xTickLabel)
        xlabel('time (ms)', 'FontWeight', 'bold')
    else
        set(gca, 'XTick', xTick, 'XTickLabel', [])
    end
    plot(t, s(i,:), 'Color', blueGreen, 'LineWidth', 1);
    plot(tAxis, [0 0], 'Color', blueGreen, 'Linewidth', 0.5)
    plot([0 0], yAxis, ...
        'Color', blueGreen, 'Linewidth', 1)
    periodPitch = 1./modFreq;
    plot([periodPitch(i) (i)], yAxis, ...
        'Color', blueGreen, 'Linewidth', 1)
    if i == 1
        text(periodPitch(i), 0.75*yAxis(2), ...
            sprintf('%s', 'period of modulation = '), ...
            'FontWeight', 'bold', ...
            'HorizontalAlignment', 'right', ...
            'VerticalAlignment', 'baseline')
    end
    plot([period(i) period(i)], yAxis, ...
        'Color', blueGreen, 'Linewidth', 1)
    text(periodPitch(i), 0.75*yAxis(2), ...
        sprintf(' %4.2f ms', 1000*periodPitch(i)), ...
        'FontWeight', 'bold', ...
        'HorizontalAlignment', 'left', ...
        'VerticalAlignment', 'baseline')
    axis([tAxis yAxis])
    ylabel('signal')
end

% Plot the spectra
fTick = 0:600:5000;
fTickLabel = fTick/1000;
fAxis = [0 2400];
aAxis = [0 1.2];
for i = 1:n
    axes('Position', pos(i+n,:))
    set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
        'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
        'FontWeight', 'bold', 'TickDir', 'out', ...
        'YTick', 0:0.5:1)
    if i == 1
        set(gca, 'XTick', fTick, 'XTickLabel', fTickLabel, ...
            'XTickLabelRotation', 0)
        xlabel('freq. (kHz)', 'FontWeight', 'bold')
    else
        set(gca, 'XTick', fTick, 'XTickLabel', [])
    end
    for j = -nrOfSidebands:nrOfSidebands
        plot(abs([fc+j*modFreq(i) fc+j*modFreq(i)]), ...
            [0 spc(i, nrOfSidebands+j+1)], ...
            'Color', blueGreen, 'Linewidth', 2)
    end
    for j = 1:5*n
        plot([j/period(i) j/period(i)], ...
            [0 0.08*amp], 'Color', red, ...
            'Linewidth', 5)
    end
    ylabel('ampl.')
    axis([fAxis aAxis])
    text(0.05*fAxis(2), 0.85*aAxis(2), ...
        sprintf('m_i = %5.2f', mi(i)), ...
        'FontWeight', 'bold', 'HorizontalAlignment', 'left')
    text(0.95*fAxis(2), 0.85*aAxis(2), ...
        sprintf('f_c = %5.0f Hz', fc), ...
        'FontWeight', 'bold', 'HorizontalAlignment', 'right')
    text(0.95*fAxis(2), 0.70*aAxis(2), ...
        sprintf('f_m = %6.0f Hz', modFreq(i)), ...
        'FontWeight', 'bold', 'HorizontalAlignment', 'right')
end

