clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Read and define the speech segment
[speech, sampleFrequency] = audioread('allLines.wav');
dt = 1/sampleFrequency;
stimulus = 0.9*speech';
t0 = 0.3471+(0:4)*0.0036;
pitchPeriod = 0.00708;
padding = 0.060;
tInterval =  [t0(1)-padding t0(end)+padding];
ti = round(tInterval(1)/dt):round(tInterval(end)/dt);
stimulusInterval = stimulus(ti);

% Save and play the sound segment
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulusInterval, sampleFrequency)
sound(stimulusInterval, sampleFrequency)

% Specify the figure parameters
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [150 240 240 120]; % pixels
xLabelHeight = 70; % pixels
yLabelWidth = 70; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
nrOfPanels = length(panelHeights);
figWidth = yLabelWidth+panelWidth+xMargin;
figHeight = 2*xLabelHeight+sum(panelHeights+yMargin);

nFft = 2048; % Number of samples in fft window
nWindow = 512;
h = hamming(nWindow)';
segment = zeros(length(t0), nFft);
for j = 1:length(t0)
    i = round(t0(j)*sampleFrequency);
    segment(j,1:length(h)) = ...
        stimulus(i-nWindow/2+1:i+nWindow/2).*h;
end

% Calculate the power spectra of the segments
spc = zeros(length(t0), nFft/2);
for j = 1:size(segment, 1)
    segment(j,(nWindow+1):nFft) = 0;
    s = abs(fft(segment(j,:))*(64/nWindow));
    spc(j,:) = 20*log10(s(1:nFft/2));
end

% Calculate the spectrogram
nOverlap = nWindow-2;
[~, f, ts, spcgr] = spectrogram(stimulusInterval, ...
    nWindow, nOverlap, nFft, sampleFrequency);
spcgr = 10*log10(abs(spcgr));
ts = ts+t0(1)-padding;

% Calculate the position of the figure on the screen
set(gcf, 'Position', [figX0 figY0 figWidth figHeight], ...
    'PaperPositionMode', 'auto', ...
    'Color', frameColor, 'InvertHardCopy', 'off')
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Calculate the plot parameters
dXTick = 0.01; % sec
tAxis = [0.8*tInterval(1)+0.2*tInterval(2) ...
    0.2*tInterval(1)+0.8*tInterval(2)];
durationStimulus = length(stimulus)*dt; % sec

% Plot the sound signal and the analysis windows
panel = 1;
axes('Position', pos(panel,:))
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:durationStimulus, ...
    'XTickLabel', 1000*(0:dXTick:durationStimulus), ...
    'YTick', -1:1)
plot((1:length(stimulus))*dt, stimulus, ...
    'Color', blueGreen, 'LineWidth', 1.5);
plot(tInterval, [0 0], ...
    'Color', blueGreen, 'Linewidth', 0.5)
for j = 1:length(t0)
    i = round(t0(j)*sampleFrequency);
    plot((i-nWindow/2+1:i+nWindow/2)*dt, h, ...
        'Color', red, 'LineWidth', 0.5);
    text(t0(j), 1.25, num2str(j), 'FontSize', 8, ...
        'Color', darkRed, 'HorizontalAlignment', 'center')
end
text(0.99*tAxis(1)+0.01*tAxis(2), 1.3, 'five analysis windows', ...
    'FontSize', 10, 'FontWeight', 'bold', 'Color', red)
text(0.99*tAxis(1)+0.01*tAxis(2), 1.0, 'signal', ...
    'FontSize', 10, 'FontWeight', 'bold', 'Color', blueGreen)
xlabel('time ms', 'FontWeight', 'bold')
ylabel('signal')
axis([tAxis -1.0 1.5])

% Plot the windowed sound signal
panel = panel+1;
axes('Position', pos(panel,:))
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:durationStimulus, ...
    'XTickLabel', [], ...
    'YTick', -1:1, 'YTickLabel', [])
for j = 1:length(t0)
    i = round(t0(j)*sampleFrequency);
    level = (j-ceil(length(t0)/2))*0.54;
    plot((i-nWindow/2+1:i+nWindow/2)*dt, ...
        level+0.5*segment(j, 1:nWindow), ...
        'Color', blueGreen, 'LineWidth', 1.5);
    plot((i-nWindow/2+1:i+nWindow/2)*dt, ...
        level+zeros(size(segment(j, 1:nWindow))), ...
        'Color', blueGreen, 'LineWidth', 0.5);
    text((i-nWindow/2)*dt, level+0.05, ...
        ['segment ' num2str(j)], ...
        'HorizontalAlignment', 'right', ...
        'VerticalAlignment', 'bottom', ...
        'FontSize', 8, 'Color', darkRed)
end
ylabel('signal')
axis([tAxis -1.5 1.5])
text(0.99*tAxis(1)+0.01*tAxis(2), 1.35, ...
    'five windowed signals', ...
    'FontSize', 10, 'FontWeight', 'bold', 'Color', blueGreen)

% Plot the spectrogram
panel = panel+1;
axes('Position', pos(panel,:))
dYTick = 1; % kHz
maxYAxis = sampleFrequency/2; % Hz
fAxis = [-20 maxYAxis+20];
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:durationStimulus, 'XTickLabel', [], ...
    'YTick', 0:1000*dYTick:sampleFrequency/2, ...
    'YTickLabel', 0:dYTick:sampleFrequency/2000)
surface(ts, f, spcgr, 'EdgeColor', 'none');
plot([tAxis(1) tAxis(2) tAxis(2) tAxis(1) tAxis(1)], ...
    [fAxis(1) fAxis(1) fAxis(2) fAxis(2) fAxis(1)], ...
    'LineWidth', 1.5, 'Color', 'k')
axis([tAxis fAxis])
caxis([max(spcgr(:))-60 max(spcgr(:))])
ylabel('frequency (kHz)', 'FontWeight', 'bold')
for j=1:size(segment, 1)
    plot([t0(j) t0(j)], fAxis, 'Color', darkRed)
end

% Plot the power spectra
panel = panel+1;
pos(panel, 3) = pos(panel, 3)/(length(t0)+0.3);
pos(panel, 2) = pos(panel, 2)+xLabelHeight/figHeight;
formants = [700 1200 1900 3500
    21   14   -3   2];
xTick = 0:1000:sampleFrequency/2;
for j = 1:size(segment, 1)
    axes('Position', pos(panel,:))
    set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
        'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
        'FontWeight', 'bold', 'TickDir', 'out', ...
        'XTick', xTick, 'XTickLabel', xTick/1000, ...
        'YTick', -120:30:120, ...
        'TickLength', [0.020 0.025])
    if j == 1
        ylabel('power (dB)')
    else
        set(gca, 'YTickLabel', [])
    end
    plot((1:length(spc(j,:)))/(nFft*dt), spc(j,:), ...
        'Color', blueGreen)
    xlabel('freq. (kHz)')
    for i = 1:length(formants)
        text(formants(1, i), formants(2, i), ...
            ['F' num2str(i)], 'Color', red, ...
            'HorizontalAlignment', 'center')
    end
    if j == 1
        text(200, 29, 'power spectrum', ...
            'FontSize', 10, 'FontWeight', 'bold', ...
            'Color', blueGreen)
        text(4600, 19, ['of segment ' num2str(j)], ...
            'FontSize', 8, 'Color', darkRed, ...
            'HorizontalAlignment', 'right')
    else
        text(4600, 29, ['of segment ' num2str(j)], ...
            'FontSize', 8, 'Color', darkRed, ...
            'HorizontalAlignment', 'right')
    end
    axis([0 4900 -40 35])
    pos(panel, 1) = pos(panel, 1) + ...
        panelWidth/figWidth/length(t0);
end

