clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Define the stimuli
sampleFrequency = 44100; % Hz
dt = 1/sampleFrequency;
durationTone = 0.5; % sec
durationInterval = 0.25; % sec
amp = 0.49;
fade = 0.01; % sec
df = [200 240 300 400 600]; % Hz
n = length(df);
fc = 1200; % Hz

% Print the difference frequencies
fLow = fc-df/2;
fHgh = fc+df/2;
period = 2./(fHgh-fLow);
fprintf('Difference frequencies: \n')
fprintf('%8.2f', fliplr(round(df, 2)))
fprintf('%s\n', '  Hz')
    
% Calculate the stimuli
t = -0.1*durationTone:dt:0.9*durationTone;
s = zeros(n, length(t));
envelope = zeros(n, length(t));
interval =  zeros(size(dt:dt:durationInterval));
stimulus = [];
for i = n:-1:1 % Start stimulus with highest modulation frequency
    envelope(i,:) = abs(2*amp*sin(2*pi*df(i)/2*t));
    s(i,:) = amp*(cos(2*pi*fHgh(i)*t)-cos(2*pi*fLow(i)*t));
    s(i,:) = FadeInOut(s(i,:), sampleFrequency, fade, fade);
    stimulus = [stimulus s(i,:) interval]; %#ok<AGROW>
end
for i = n:-1:1 % Start stimulus with highest modulation frequency
    tone = amp*sin(2*pi*(fHgh(i)-fLow(i))*t);
    tone = FadeInOut(tone, sampleFrequency, fade, fade);
    stimulus = [stimulus tone interval]; %#ok<AGROW>
end

% Calculate the spectra
spc = zeros(n, 2);
for i = 1:n
    spc(i,:) = [fLow(i) fHgh(i)];
end

% Save and play the stimulus
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = [480 225]; % pixels
panelHeights = 120*ones(1, n); % pixels
xLabelHeight = 70; % pixels
yLabelWidth = [70 70]; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the signals
dXTick = 0.0025; % sec
for i = 1:n
    axes('Position', pos(i,:))
    tAxis = [-0.125*period(1) 2.875*period(1)]/2;
    yAxis = [-1.2 1.5];
    xTick = -dXTick:dXTick:durationTone;
    xTickLabel = xTick;
    set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
        'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
        'FontWeight', 'bold', 'TickDir', 'out', ...
        'YTick', -1:1)
    if i == 1
        set(gca, 'XTick', xTick, 'XTickLabel', 1000*xTickLabel)
        xlabel('time (ms)', 'FontWeight', 'bold')
    else
        set(gca, 'XTick', xTick, 'XTickLabel', [])
    end
    plot(t, s(i,:), 'Color', blueGreen, 'LineWidth', 1.5);
    plot(t, envelope(i,:), '--', 'Color', red, 'LineWidth', 1);
    plot(t, -envelope(i,:), '--', 'Color', red, 'LineWidth', 1);
    plot(tAxis, [0 0], 'Color', blueGreen, 'Linewidth', 0.5)
    plot([0 0], yAxis,'Color', blueGreen, 'Linewidth', 2)
    plot([period(i) period(i)], yAxis, ...
        'Color', blueGreen, 'Linewidth', 2)
    if i == 1
        text(period(i), 0.85*yAxis(2), ...
            sprintf('%s', 'period of signal = '), ...
            'HorizontalAlignment', 'right')
    end
    text(period(i), 0.85*yAxis(2), ...
        sprintf(' %4.1f ms', 1000*period(i)), ...
        'HorizontalAlignment', 'left')
    axis([tAxis yAxis])
    ylabel('signal')
end
panelWidth = 2;

% Plot the spectra
fTick = 0:400:5000;
fTickLabel = fTick/1000;
fAxis = [0 1900];
aAxis = [0 1.2];
for i = 1:n
    axes('Position', pos(i+n,:))
    set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
        'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
        'FontWeight', 'bold', 'TickDir', 'out', ...
        'YTick', 0:0.5:1)
    if i == 1
        set(gca, 'XTick', fTick, 'XTickLabel', fTickLabel)
        xlabel('freq. (kHz)', 'FontWeight', 'bold')
    else
        set(gca, 'XTick', fTick, 'XTickLabel', [])
    end
    plot([fLow(i) fLow(i)], [0 amp], [fHgh(i) fHgh(i)], [0 amp], ...
        'Color', blueGreen, 'Linewidth', 2)
    for j = 1:5*n
        plot([j/period(i) j/period(i)], ...
            [0 0.1*amp], 'Color', red, ...
            'Linewidth', 3)
    end
    ylabel('ampl.')
    axis([fAxis aAxis])
    text(0.05*fAxis(2), 0.85*aAxis(2), ...
        sprintf('1/period = %3.0f Hz', 1/period(i)), ...
        'FontWeight', 'bold')
    text(0.95*fAxis(2), 0.85*aAxis(2), ...
        sprintf('f_1 = %5.0f Hz', fLow(i)), ...
        'FontWeight', 'bold', 'HorizontalAlignment', 'right')
    text(0.95*fAxis(2), 0.70*aAxis(2), ...
        sprintf('f_2 = %5.0f Hz', fHgh(i)), ...
        'FontWeight', 'bold', 'HorizontalAlignment', 'right')
end


