clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Calculate the Doppler frequency
sampleFrequency = 44100; % Hz
dt = 1/sampleFrequency; % sec
speedOfSound = 343; % m/s
speedOfSource = 100; % m/s
duration = 4; % sec 
t = -0.5*duration:dt:0.5*duration; % sec
r = speedOfSource*t; % meter
passingDistance = 2; % meter
dist = sqrt(passingDistance*passingDistance+r.*r);
f0 = 440; % Hz
nrOfHarmonics = 10;
costh = r./dist;
fDoppler = ...
    f0 * speedOfSound./(speedOfSound + speedOfSource*costh);
amp = 0.9*min(dist)./dist;

% Model the Doppler illusion by adding a gaussian distribution.
maxInc = 200; % Hz
m = -0.002; % sec
sdLeft = 0.015; % sec
sdRight = 0.005; % sec
varLeft = sdLeft*sdLeft;
varRight = sdRight*sdRight;
iLeft = find(t<=m);
iRight = find(t>m);
illusion(iLeft) = ...
    maxInc*exp(-(t(iLeft)-m).*(t(iLeft)-m)/(2*varLeft));
illusion(iRight) = ...
    maxInc*exp(-(t(iRight)-m).*(t(iRight)-m)/(2*varRight));

% Calculate the stimulus
phi = 2*pi*cumsum(fDoppler)*dt;
stimulus = sin(phi);
for k = 2:nrOfHarmonics
    stimulus = stimulus + (1/k)*sin(k*phi);
end
stimulus = (2/pi)*amp.*stimulus;
stimulus = FadeInOut(stimulus, sampleFrequency, 0.02, 0.02);

% Save and play wav file
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [200 200 220]; % pixels
xLabelHeight = [70 0 70]; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the position of the panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the Doppler frequency
panel = 1;
axes('Position', pos(panel,:))
dXTick = 0.05;
xTick = -1:dXTick:1;
xRange = [-0.16 0.16];
yRange = [300 750];
set(gca, 'Box', 'on', 'LineWidth', 2, ...
    'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'XTick', xTick, 'YTick', 0:100:10000, ...
    'FontSize', 14, 'Fontweight', 'bold')
plot(t, fDoppler+illusion, ':', 'LineWidth', 2, 'Color', red)
plot(t, fDoppler, 'LineWidth', 2, 'Color', blueGreen)
plot(xRange, [f0 f0], 'Color', darkRed)
plot([0 0], yRange, 'Color', darkRed) 
xlabel('time (s)', 'FontWeight', 'bold', 'FontSize', 18)
ylabel('{\it{F}}_{0} (Hz)', 'FontWeight', 'bold')
axis([xRange yRange])

% Plot the amplitude
panel = panel+1;
axes('Position', pos(panel,:))
ampdB = 20*log10(amp);
yRange = [-22 3];
set(gca, 'Box', 'on', 'LineWidth', 2, ...
    'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'XTick', xTick, 'XTickLabel', [], ...
    'YTick', -100:10:20, ...
    'FontSize', 14, 'Fontweight', 'bold')
plot(t, ampdB, 'LineWidth', 2, 'Color', blueGreen)
plot(xRange, [f0 f0], 'Color', darkRed)
plot([0 0], yRange, 'Color', darkRed) 
ylabel('level (dB)', 'FontWeight', 'bold', 'FontSize', 18)
axis([xRange yRange])

% Plot the scheme
panel = panel+1;
axes('Position', pos(panel,:))
dXTick = 5; % meter
xTick = r(1):dXTick:r(end);
xRange = xRange*speedOfSource;
yRange = [-3 7];
r = -10;
pArrow = 8;
lArrow = 1.5;
wArrow = 0.3;
set(gca, 'Box', 'on', 'LineWidth', 2, ...
    'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'XTick', xTick, 'YTick', -10:2:10, ...
    'FontSize', 14, 'Fontweight', 'bold')
plot([xRange(1) xRange(end)], ...
    [passingDistance passingDistance], ...
    'LineWidth', 2, 'Color', blueGreen)
plot(0, 0, 'o', ...
    'LineWidth', 2, 'MarkerSize', 18, 'Color', blueGreen)
plot([r 0], [passingDistance 0], 'Color', red)
plot([r r], passingDistance+[-0.2 0.2], ...
    'LineWidth', 2, 'Color', blueGreen)
plot([0 0], passingDistance+[-0.2 0.2], ...
    'LineWidth', 2, 'Color', blueGreen)
plot([pArrow-lArrow pArrow pArrow-lArrow], ...
    passingDistance +[wArrow 0 -wArrow], ...
    'LineWidth', 2, 'Color', blueGreen)
% plot the circles representing the stimulus
nrOfCircles = 5;
periods = (1:nrOfCircles)/f0;
yCntr = passingDistance;
xCntr = r-speedOfSource*periods;
radius = speedOfSound*periods;
for k = 1:nrOfCircles
    p = CalcRegularPolygon(xCntr(k), yCntr, radius(k), 0, 100);
    PlotPolygon(p, red, 1)
end
text(pArrow, passingDistance, 'v_{src}', ...
    'FontSize', 14, 'VerticalAlignment', 'Bottom')
text(0.4*r, passingDistance, 'r', 'FontSize', 14, ...
    'VerticalAlignment', 'Bottom')
text(r+4, passingDistance+0.18, '\theta', ...
    'FontWeight', 'bold', 'FontSize', 14, ...
    'VerticalAlignment', 'Top')
xlabel('x (m)', 'FontWeight', 'bold', 'FontSize', 18)
ylabel('y (m)', 'FontWeight', 'bold', 'FontSize', 18)
axis([xRange yRange])


