% % copyright statement
% % overview

% PSS/mScripts/loudness_EqualLoudnessContour.m

clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Calculate the loudness functions from z_min to z_max
I_ref = 1.0e-12;
z_min = Hz2ERB(50);
z_max = Hz2ERB(20000);
dz = 0.1;
z = z_min:dz:z_max;
I_dB = -10:120;
f_ref = 1000; % Hz
loudnessEq = zeros(size(I_dB));
for k = 1:length(I_dB)
    I = I_ref*10.^(I_dB(k)/10);
    [~, specificLoudness] = ...
        CalcLoudnessFunctions(f_ref, I, z, 1, 1);
    loudnessEq(k) = sum(specificLoudness)*dz;
end

% Interpolate the loudness for loudness levels cL = 0:10:120
cL = 0:10:120;
loudnessLevels = interp1(I_dB, loudnessEq, cL);
fMin = 60;
fMax = 15000;
f = logspace(log10(fMin), log10(fMax), 100);
equalLC = zeros(length(cL), length(f));
for i = 1:length(f)
    loudnessEq = zeros(size(I_dB));
    for k = 1:length(I_dB)
        I = I_ref*10.^(I_dB(k)/10);
        [~, specificLoudness] = ...
            CalcLoudnessFunctions(f(i), I, z, 1, 1);
        loudnessEq(k) = sum(specificLoudness)*dz;
    end
    equalLC(:, i) = interp1(loudnessEq, I_dB, loudnessLevels);
end

% Resample the equal loudness contour of 60 dB, for which k60 = 7
% in order to convert it to the amplitudes of the FM tone.
k60 = 7; % Adapt when cL is changed
sampleFrequency = 44100; % Hz
dt = 1/sampleFrequency;
durationTone = 20; % sec
t = dt:dt:durationTone;
maxElc = max((equalLC(k60,:)));
amplSamples_dB = equalLC(k60,:)-maxElc;
resampleFreqs = logspace(log10(fMin), log10(fMax), length(t));
amplResampled_dB = spline(log10(f), amplSamples_dB, log10(resampleFreqs));
amplResampled = 10.^(amplResampled_dB/20);
% Calculate the FM tone linearly modulated on a logarithmic frequency scale
tau = durationTone/(log(fMax)-log(fMin)); % sec
phi = 2*pi*tau*fMin*(exp(t/tau)-1);
tone = sin(phi);
% Avoid audible clicks at onset and offset
tone = FadeInOut(tone, sampleFrequency, 0.02, 0.02);
toneEqualLoudness = amplResampled.*tone;
meanAmplitude = mean(amplResampled);
meanAmplitude_dB = 20*log10(meanAmplitude)+maxElc;
toneEqualAmplitude = meanAmplitude*tone;
% Make the stimulus consisting of the FM tone of equal loudness, a
% 2-seconds pause, and the FM tone of equal amplitude
stimulus = ...
    [toneEqualLoudness zeros(1, 2*sampleFrequency) toneEqualAmplitude];

% Save and play wav file
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Calculate the loudness of the FM tone of equal loudness
I_s_dB = equalLC(k60,:);
loudnessL = zeros(size(f));
I_s = zeros(size(f));
for i = 1:length(f)
    I_s(i) = I_ref*10^(I_s_dB(i)/10);
    [~, specificLoudness] = ...
        CalcLoudnessFunctions(f(i), I_s(i), z, 1, 1);
    loudnessL(i) = sum(specificLoudness)*dz;
end

% Calculate the loudness of the FM tone of equal amplitude
I_s_dB = meanAmplitude_dB;
loudnessI = zeros(size(f));
for i = 1:length(f)
    I_s = I_ref*10^(I_s_dB/10);
    [~, specificLoudness] = ...
        CalcLoudnessFunctions(f(i), I_s, z, 1, 1);
    loudnessI(i) = sum(specificLoudness)*dz;
end

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [300 300]; % pixels
xLabelHeight = [70 60]; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the level of the glides in dB SPL
panel = 1;
axes('Position', pos(panel,:))
xTick = [20 50 100 200 500 1000 2000 5000 10000 20000];
set(gca, 'Box', 'on', 'LineWidth', 2, ...
    'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'XTick', log10(xTick), 'XTickLabel', xTick/1000, ...
    'YTick', -20:10:120, ...
    'FontSize', 14, 'Fontweight', 'bold', 'TickDir', 'out')
plot(log10(f), equalLC(k60, :), ...
    'LineWidth', 2,  'Color', blueGreen)
plot([log10(f(1)) log10(f(end))], [meanAmplitude_dB meanAmplitude_dB], ...
    ':', 'LineWidth', 2,  'Color', red)
text(log10(f(1)+3), equalLC(k60, 1)+1, '1: equal loudness', ...
    'FontSize', 12, 'Fontweight', 'bold',  'Color', blueGreen)
text(log10(f(1)), meanAmplitude_dB-2, '2: equal amplitude', ...
    'FontSize', 12, 'Fontweight', 'bold',  'Color', red)
xlabel('frequency (kHz)', 'FontSize', 14, 'Fontweight', 'bold')
ylabel('level (dB)', 'FontSize', 14, 'Fontweight', 'bold')
axis([log10(50) log10(18000) 48 90])

% Plot the loudnes of the glides in sone
panel = panel+1;
axes('Position', pos(panel,:))
ti = 0:4:durationTone;
xTick = log10(fMin)+(log10(fMax)-log10(fMin))*ti/durationTone;
yTick = [0.25 0.5 1 2 4 8 16];
set(gca, 'Box', 'on', 'LineWidth', 2, ...
    'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'XTick', xTick, 'XtickLabel', ti, ...
    'YTick', log10(yTick), 'YtickLabel', yTick, ...
    'FontSize', 14, 'Fontweight', 'bold')
plot(log10(f), log10(loudnessL), ...
    'LineWidth' , 2, 'Color', blueGreen)
plot(log10(f), log10(loudnessI), ...
    ':', 'LineWidth' , 2, 'Color', red)
text(log10(f(1)), log10(loudnessL(1)+1), '1: equal loudness', ...
    'FontSize', 12, 'Fontweight', 'bold',  'Color', blueGreen)
text(log10(f(1)+4), log10(loudnessI(1)), '2: equal amplitude', ...
    'FontSize', 12, 'Fontweight', 'bold',  'Color', red)
xlabel('time (s)')
ylabel('loudness (sone)')
axis([log10(50) log10(18000) log10(0.5) log10(20)])

