clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Define and calculate the sound signal
[speech, sampleFrequency] = audioread('allLines.wav');
stimulus = 0.9*speech';
dt = 1/sampleFrequency;
durationStimulus = length(stimulus)*dt; % sec
t = (0:length(stimulus))*dt;

% Save and play the stimulus
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, 0.2*stimulus, sampleFrequency)
sound(0.2*stimulus, sampleFrequency)

% Calculate the sound pressure levels for different time constants 'tau'
tau = [0.035 0.125 1.000]; % sec
intensity = zeros(length(tau), length(stimulus));
a = zeros(size(tau));
for k = 1: length(tau)
    a(k) = exp(-dt/tau(k));
    intensity(k,:) = (1-a(k))*filter(1, [1 -a(k)], stimulus.*stimulus);
end

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [120 120 120 120 120]; % pixels
xLabelHeight = 70; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the positions of panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot the speech signal
panel = 1;
axes('Position', pos(panel,:))
dXTick = 0.5; % sec
dYTick = 1; % kHz
tAxis = [t(1)-dXTick t(end)+dXTick];
maxYAxis = sampleFrequency/2; % Hz
fAxis = [0 maxYAxis];
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:durationStimulus, ...
    'XTickLabel', 0:dXTick:durationStimulus, ...
    'YTick', -1:1)
plot((1:length(stimulus))/sampleFrequency, stimulus, ...
    'Color', blueGreen, 'LineWidth', 1.5);
axis([0 durationStimulus -1.5 1.5])
plot(tAxis, [0 0], 'Color', blueGreen, 'Linewidth', 0.5)
xlabel('time (s)')
ylabel('signal')
text(0.02*durationStimulus, 1.2, 'speech utterance', ...
    'FontSize', 10, 'FontWeight', 'bold')
syllables = ...
    ['All  '; 'lines'; 'from '; 'Lon -'; 'don  '; 'are  '; 'en  -'; 'gaged'];
onsets = [0.13 0.34 0.70 0.90 1.10 1.25 1.40 1.60];
for syll = 1:length(syllables)
    text(onsets(syll), -0.8, syllables(syll,:), 'FontAngle', 'italic', ...
        'FontSize', 10, 'FontWeight', 'bold', 'Color', blueGreen)
end

% Plot the squared speech signal
panel = panel+1;
axes('Position', pos(panel,:))
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', 0:dXTick:durationStimulus, ...
    'XTickLabel', [], ...
    'YTick', -1:0.5:2)
plot((1:length(stimulus))*dt, stimulus.*stimulus, ...
    'Color', blueGreen, 'LineWidth', 1.5);
plot(tAxis, [0 0], ...
    'Color', blueGreen, 'Linewidth', 0.5)
ylabel('signal')
axis([0 durationStimulus -0.4 1.4])
text(0.02*durationStimulus, 1.2, 'squared speech utterance', ...
    'FontSize', 10, 'FontWeight', 'bold')

% Plot the measured sound pressure levels as a function of time
tau = [0.035 0.125 1.000]; % sec
for k = 1:length(tau)
    panel = panel+1;
    axes('Position', pos(panel,:))
    set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
        'Box', 'on', 'NextPlot', 'add', 'Color', lightYellowGreen, ...
        'FontWeight', 'bold', 'TickDir', 'out', ...
        'XTick', 0:dXTick:durationStimulus, 'XTickLabel', [], ...
        'YTick', -100:20:100, 'YTickLabel', -100:20:100)
    plot((1:length(stimulus))*dt, 10*log10(intensity(k,:)), ...
        'Color', blueGreen, 'LineWidth', 1.5);
    ylabel('SPL (dB)')
    axis([0 durationStimulus -50 0])
    % Calculate and plot the decaying exponential
    n = 3*round(tau(k)/dt);
    zn = round(durationStimulus/30*sampleFrequency);
    imprsp = 0.7*a(k).^(1:n);
    plot((0.4*sampleFrequency+(1:length(imprsp)+zn))*dt-zn*dt, ...
        -45+20*[zeros(1, zn) imprsp], ...
        'Color', blueGreen);
    txtTau = sprintf(' = %5.3f s', tau(k));
    text(0.14, -42, '\tau ', ...
        'FontSize', 15, 'FontWeight', 'bold', ...
        'VerticalAlignment', 'bottom')
    text(0.17, -42, txtTau, ...
        'FontSize', 12, 'FontWeight', 'bold', ...
        'VerticalAlignment', 'bottom')
    switch k
        case 1
            text(2.05, -10, '''IMPULSE''', ...
                'FontSize', 12, 'FontWeight', 'bold', ...
                'HorizontalAlignment', 'right')
        case 2
            text(2.05, -10, '''FAST''', 'FontSize', 12, ...
                'FontSize', 12, 'FontWeight', 'bold', ...
                'HorizontalAlignment', 'right')
        case 3
            text(2.05, -10, '''SLOW''', 'FontSize', 12, ...
                'FontSize', 12, 'FontWeight', 'bold', ...
                'HorizontalAlignment', 'right')
    end
end


