clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Calculate the pulse series
sampleFrequency = 22050; % Hz
dt = 1/sampleFrequency;
durationNoise = 0.1; % sec
durationPulses = 0.1; % sec
durationInterval = 0.01; % sec
tPulses = dt:dt:durationPulses;
tInterval = dt:dt:durationInterval;
interval = zeros(size(tInterval));
f0 = 30; % Hz
p = round((dt:1/f0:durationPulses)*sampleFrequency)+1;
pulses(p) = 0.9;
delay = 0.005; % sec
shift = round(delay*sampleFrequency);
pulsePairs = pulses + circshift(pulses, [0, shift]);
pulsePairs = FadeInOut(pulsePairs, sampleFrequency, 0.01, 0.01);
stimulus = [interval pulsePairs interval];
t = (1:length(stimulus))*dt;
t0 = 0.05; % Moment at which pitch is estimated, in seconds
segmentDuration = 0.060; % sec
segment = [t0-0.5*segmentDuration t0+0.5*segmentDuration];
sI = round(segment*sampleFrequency);
stimulusSegment = ...
    stimulus(sI(1):sI(2));
stimulusSegment = ...
    FadeInOut(stimulusSegment, sampleFrequency, 0.005, 0.005);

% Save and play the stimulus segment
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulusSegment, sampleFrequency)
sound(stimulusSegment, sampleFrequency)

% Calculate and apply dBA filtering
[b_dBA, a_dBA] = Calculate_dBAFilter(sampleFrequency);
stimulusDBA = filter(b_dBA, a_dBA, stimulus);

% Calculate the center frequencies of the gammatone filterbank
numberOfFilters = 128;
minimumFrequency = 100;
maximumFrequency = sampleFrequency/2;
centerFrequencies = ...
    CalculateCenterFrequencies(numberOfFilters, ...
    minimumFrequency, maximumFrequency);
% Calculate the filter coefficients of the gammatone filter bank
[b, a] = ...
    CalculateGammatoneFilterBank(sampleFrequency, centerFrequencies);

% Band-pass filtering
basilarMembrane = zeros(numberOfFilters, length(stimulusDBA));
for k = 1:numberOfFilters
    basilarMembrane(k,:) = filter(b(k,:), a(k,:), stimulusDBA);
end

% Half-wave rectification and compression
cochleogram = (basilarMembrane+abs(basilarMembrane))/2;
cochleogram = cochleogram.^(1/3);

% Low-pass filtering
cutoffFrequency = 3000; % Hz
[b, a] = butter(4, 2*cutoffFrequency/sampleFrequency);
for k = 1:numberOfFilters
    cochleogram(k,:) = filter(b, a, cochleogram(k,:));
end

% Pitch estimation
maxPitch = 3000; % maximum pitch in Hertz
minPitch = 33; % minimum pitch in Hertz
maxDelay = 1/minPitch; % sec;
n = round(maxDelay*sampleFrequency);
% High-pass filter to remove tilt from summary autocovariance
cutoffFrequency = minPitch; % Hz
[b, a] = butter(4, 2*cutoffFrequency/sampleFrequency, 'high');

% Calculate the autocovariances at t0
n1 = round(t0*sampleFrequency)-n;
n2 = n1+2*n;
nFft = 2^nextpow2(2*n-1);
h = hanning(2*n+1);
hann = repmat(h', numberOfFilters, 1);
acv = zeros(numberOfFilters, 2*n+1);
frames(:,1:2*n+1) = ...
    hann.*cochleogram(:,n1:n2);
for k = 1:numberOfFilters
    x = frames(k,:)-mean(frames(k,:));
    X = fft(x, nFft);
    ac = ifft(X.*conj(X))/nFft;
    acv(k,2*n+1:-1:1) = ac([nFft-n+1:nFft 1:n+1]);
end
sav = mean(acv);
sav = filtfilt(b, a, sav);

% Determine the pitch peak in the autocovariance function
mx = 0;
T0 = n+1;
g1 = n+2;
while sav(g1) > sav(g1-1)
    g1 = g1+1;
end
for g = g1:2*n-2
    if sav(g-1) < sav(g) && ...
            sav(g) > sav(g+1)
        if sav(g) > mx && sav(g) > 0.2*sav(n+1)
            % Parabolic interpolation of T0 and pitch strength
            sd = sav(g-1)-sav(g+1);
            se = sav(g-1)-2*sav(g)+sav(g+1);
            T0 = g+0.5*sd/se;
            pitchStrength = sav(g)-0.125*sd*sd/se;
            mx = pitchStrength;
        end
    end
end
f0 = sampleFrequency/(T0-n-1);

% Calculate the position of the figure on the screen
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [180 300 120]; % pixels
xLabelHeight = [70 0 70]; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeight, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the position of the panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeight, ...
    xMargin, yMargin);

% Plot summary autocovariance
panel = 1;
axes('Position', pos(panel,:))
dXTick = 0.01;
xTicklabel = -0.05:dXTick:0.05;
tauTick = ...
    interp1([-maxDelay maxDelay], [1 2*n+1], xTicklabel, ...
    'linear', 'extrap');
maxSac = max(sav);
yTick = [-maxSac 0 maxSac];
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', tauTick, 'XTickLabel', 1000*xTicklabel, ...
    'YTick', yTick, 'YTickLabel', ['   ';' 0 '; 'max'], ...
    'NextPlot', 'add')
plot(1:length(sav), sav, 'LineWidth', 2, 'Color', blueGreen)
plot([1 length(sav)], [0 0], 'Color', [0 0 0])
xlabel('delay (ms)')
ylabel('summ. autocov.')
axis([1 2*n+1 -0.5*maxSac 1.3*maxSac])
% Indicate the pitch peak in the autocovariance function
if T0>n+1
    plot([T0 T0], [-3*maxSac 3*maxSac], 'Color', darkRed)
end
text(T0, -0.3*maxSac, ...
    sprintf('estimated pitch period: '), ...
    'FontSize', 12, 'FontWeight', 'bold', ...
    'HorizontalAlignment', 'right')
text(T0, -0.3*maxSac, ...
    sprintf('%5.2f ms', 1000*(T0-n-1)*dt), ...
    'FontSize', 12, 'FontWeight', 'bold', ...
    'HorizontalAlignment', 'left')

% Plot autocovariance
panel = panel+1;
axes('Position', pos(panel,:))
yTicklabel = [50 100 200 500 1000 2000 5000 10000 20000];
yTick = ...
    spline(centerFrequencies, 1:numberOfFilters, yTicklabel);
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', tauTick, 'XTickLabel', [], ...
    'YTick', yTick, 'YTickLabel', yTicklabel/1000, ...
    'NextPlot', 'add')
surface(1:2*n+1, 1:numberOfFilters, acv(:, 1:2*n+1), 'EdgeColor','none');
plot([1 2*n+1 2*n+1 1], [1 1  numberOfFilters numberOfFilters], 'k', ...
    'LineWidth', 1.5)
ylabel('frequency (kHz)')
axis([1 2*n+1 1 numberOfFilters])
caxis([-max(acv(:)) max(acv(:))])

% Plot the sound signal
panel = panel+1;
axes('Position', pos(panel,:))
xTick = 0:dXTick:t(end);
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', xTick, 'XTickLabel', 1000*xTick, ...
    'YTick', -1:1, 'YTickLabel', -1:1, ...
    'NextPlot', 'add')
plot(t, stimulus, 'LineWidth', 2, 'Color', blueGreen)
plot([t(1) t(end)], [0 0], 'Color', [0 0 0])
xlabel('time (ms)')
ylabel('signal')
axis([t0-maxDelay t0+maxDelay -1.1 1.1])


