clear
close all
InitializePSS()
fprintf('%s\n', mfilename);

% Specify the tones
sampleFrequency = 22050; % Hz
dt = 1/sampleFrequency;
durationTones = 0.1; % sec
durationInterval = 0.02; % sec
tTones = dt:dt:durationTones;
tInterval = dt:dt:durationInterval;
interval = zeros(size(tInterval));
f0 = 200; % Hz
rankOfHarmonics = 9:11;
amp = 0.9/length(rankOfHarmonics);
fade = 0.01; % sec
t0 = 0.054; % Moment at which pitch is estimated, in seconds
segmentDuration = 0.060; % Duration in seconds of segment
% for which pitch is estimated
segment = [t0-0.5*segmentDuration t0+0.5*segmentDuration];

% Calculate the stimuli
tone = amp*sin(2*pi*rankOfHarmonics(1)*f0*(tTones-1));
for j = 2:length(rankOfHarmonics)
    tone = tone + amp*sin(2*pi*rankOfHarmonics(j)*f0*(tTones-1));
end
tone = FadeInOut(tone, sampleFrequency, fade, fade);
stimulus = [interval tone interval];
env = amp*(1+2*cos(2*pi*f0*(tTones-1)));
envelope= [interval env interval];
t = (1:length(stimulus))*dt;

% Save and play the stimulus
wavFileName = sprintf('wavFiles/%s.wav', mfilename);
audiowrite(wavFileName, stimulus, sampleFrequency)
sound(stimulus, sampleFrequency)

% Calculate the dBA filter
[b_dBA, a_dBA] = Calculate_dBAFilter(sampleFrequency);

% Calculate the center frequencies of the gammatone filterbank
numberOfFilters = 128;
minimumFrequency = 100;
maximumFrequency = sampleFrequency/2;
centerFrequencies = ...
    CalculateCenterFrequencies(numberOfFilters, ...
    minimumFrequency, maximumFrequency);
% Calculate the filter coefficients of the gammatone filter bank
[b, a] = ...
    CalculateGammatoneFilterBank(sampleFrequency, centerFrequencies);

% Determine 3000-Hz low-pass filter parameters
cutoffFrequency = 3000; % Hz
[bLowPass, aLowPass] = butter(4, 2*cutoffFrequency/sampleFrequency);

% Pitch estimation parameters
maxPitch = 3000; % maximum pitch in Hertz
minPitch = 33; % minimum pitch in Hertz
maxDelay = 1/minPitch; % sec;
n = round(maxDelay*sampleFrequency);

% Calculate 33-Hz high-pass filter parameters
% to remove tilt from summary autocovariance
cutoffFrequency = minPitch; % Hz
[bHigh, aHigh] = butter(4, 2*cutoffFrequency/sampleFrequency, 'high');

% Calculate the analysis windows of the auditory filter outputs
n1 = round(t0*sampleFrequency)-n;
n2 = n1+2*n;
nFft = 2^nextpow2(2*n-1);
h = hanning(2*n+1);
hann = repmat(h', numberOfFilters, 1);

% Apply the dBA filter
stimulusDBA = filter(b_dBA, a_dBA, stimulus);

% Band-pass filtering
basilarMembrane = zeros(numberOfFilters, length(stimulusDBA));
for j = 1:numberOfFilters
    basilarMembrane(j,:) = filter(b(j,:), a(j,:), stimulusDBA);
end

% Half-wave rectification and compression
cochleogram = (basilarMembrane+abs(basilarMembrane))/2;
cochleogram = cochleogram.^(1/3);

% Low-pass filtering
for j = 1:numberOfFilters
    cochleogram(j,:) = ...
        filter(bLowPass, aLowPass, cochleogram(j,:));
end

% Pitch estimation
% Calculate the autocorrelograms at t0
acv = zeros(numberOfFilters, 2*n+1);
frames(:,1:2*n+1) = ...
    hann.*cochleogram(:,n1:n2);
for j = 1:numberOfFilters
    x = frames(j,:)-mean(frames(j,:));
    X = fft(x, nFft);
    ac = ifft(X.*conj(X))/nFft;
    acv(j,2*n+1:-1:1) = ac([nFft-n+1:nFft 1:n+1]);
end
sav = mean(acv);
sav = filtfilt(bHigh, aHigh, sav);

% Determine the pitch peak in the autocovariance function
mx = 0;
T0 = n+1;
g1 = n+1;
while sav(g1) > 0
    g1 = g1+1;
end
for g = g1:2*n-2
    if sav(g-1) < sav(g) && sav(g) > sav(g+1)
        if sav(g) > mx && sav(g) > 0.2*sav(n+1)
            % Parabolic interpolation of T0(k) and pitch strength
            sd = sav(g-1)-sav(g+1);
            se = sav(g-1)-2*sav(g)+sav(g+1);
            T0 = g+0.5*sd/se;
            pitchStrength = sav(g)-0.125*sd*sd/se;
            mx = pitchStrength;
        end
    end
end
pitchEstimate = sampleFrequency/(T0-n-1);
fprintf('Estimated pitch: %8.2f\n', pitchEstimate)

% Figure parameters
figX0 = 270; % horizontal position of figure in pixels from left
figY0 = 120; % vertical position figure in pixels from bottom
panelWidth = 720; % pixels
panelHeights = [180 300 120]; % pixels
xLabelHeights = [70 0 70]; % pixels
yLabelWidth = 90; % pixels
xMargin = 15; % pixels
yMargin = 15; % pixels

% Calculate the position of the figure on the screen
[figWidth, figHeight] = SetGcf(figX0, figY0, ...
    panelWidth, panelHeights, ...
    xLabelHeights, yLabelWidth, xMargin, yMargin, ...
    frameColor);
% Calculate the position of the panels within the figure
pos = CalculatePanelPositions(figWidth, figHeight, ...
    panelWidth, panelHeights, yLabelWidth, xLabelHeights, ...
    xMargin, yMargin);

% Plot summary autocovariance
panel = 1;
axes('Position', pos(panel,:))
dXTick = 0.005;
xTicklabel = -0.05:dXTick:0.05;
tauTick = ...
    interp1([-maxDelay maxDelay], [1 2*n+1], xTicklabel, ...
    'linear', 'extrap');
maxSac = max(sav);
yTick = [-maxSac 0 maxSac];
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', tauTick, 'XTickLabel', 1000*xTicklabel, ...
    'YTick', yTick, 'YTickLabel', ['   ';' 0 '; 'max'], ...
    'NextPlot', 'add')
xRange = ...
    round([0.75 + 0.25*length(sav) 0.25 + 0.75*length(sav)]);
plot(xRange(1):xRange(end), sav(xRange(1):xRange(end)), ...
    'LineWidth', 2, 'Color', blueGreen)
plot([1 length(sav)], [0 0], 'Color', [0 0 0])
xlabel('delay (ms)')
ylabel('summ. autocov.')
axis([xRange -1.1*maxSac 1.1*maxSac])
% Indicate the pitch peak in the autocovariance function
if T0>n+1
    plot([T0 T0], [-3*maxSac 3*maxSac], ...
        'Color', darkRed)
end
text(T0, -0.9*maxSac, ...
    sprintf('estimated pitch period: '), ...
    'FontSize', 12, 'FontWeight', 'bold', ...
    'HorizontalAlignment', 'right')
text(T0, -0.9*maxSac, ...
    sprintf('%5.2f ms', 1000*(T0-n-1)*dt), ...
    'FontSize', 12, 'FontWeight', 'bold', ...
    'HorizontalAlignment', 'left')

% Plot autocovariance
panel = panel+1;
axes('Position', pos(panel,:))
yTicklabel = [50 100 200 500 1000 2000 5000 10000 20000];
yTick = ...
    spline(centerFrequencies, 1:numberOfFilters, yTicklabel);
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', tauTick, 'XTickLabel', [], ...
    'YTick', yTick, 'YTickLabel', yTicklabel/1000, ...
    'NextPlot', 'add')
surface(xRange(1):xRange(end), 1:numberOfFilters, ...
    acv(:, xRange(1):xRange(end)), 'EdgeColor', 'none');
plot([xRange(1) xRange(end) xRange(end) xRange(1)], ...
    [1 1  numberOfFilters numberOfFilters], 'k', 'LineWidth', 1.5)
ylabel('frequency (kHz)')
axis([xRange(1) xRange(end) 1 numberOfFilters])
caxis([-max(acv(:)) max(acv(:))])

% Plot the sound signal
panel = panel+1;
axes('Position', pos(panel,:))
xTick = 0:dXTick:t(end);
set(gca, 'LineWidth', 1.5, 'FontSize', 14, ...
    'Box', 'on', 'Color', lightYellowGreen, ...
    'FontWeight', 'bold', 'TickDir', 'out', ...
    'XTick', xTick, 'XTickLabel', 1000*xTick, ...
    'YTick', -1:1, 'YTickLabel', -1:1, ...
    'NextPlot', 'add')
plot(t, stimulus, 'LineWidth', 2, 'Color', blueGreen)
plot(t, envelope, '--', 'Color', red)
plot(t, -envelope, '--', 'Color', red)
plot([t(1) t(end)], [0 0], 'Color', [0 0 0])
xlabel('time (ms)')
ylabel('signal')
% Plot the middle half of the segment
axis([t0-0.5*maxDelay t0+0.5*maxDelay -1.1 1.1])

